/*global require: false */
require(["dojo/parser",
         "dojo/dom",
         "dojo/dom-construct",
         "dojo/json",
         "dojo/on",
         "dojo/_base/lang",       
         "dijit/form/Button",
         "dijit/form/Select",
         "dijit/form/ValidationTextBox",
         "dijit/registry",
         "dojox/layout/TableContainer",
         "dojox/rpc/Service",
         "dojox/rpc/JsonRPC",
         "dojo/ready"],
function(parser,dom,domConstruct,json,on,lang,
		Button,Select,ValidationTextBox,registry,
		TableContainer,Service,JsonRPC,ready) {
	
ready(function() {
	
	var jsonRpcDemo = new Service("/jsonBeanDemo?smd");
	
	var methodSelect = registry.byId("methodSelect");
	var sendBtn = registry.byId("sendBtn");

	methodSelect.addOption({value:"",label:"\u2014select a function\u2014"});

	for (var m in jsonRpcDemo) {
		
		var mm = jsonRpcDemo[m];
		
		if (mm.servicePath && lang.isFunction(mm)) {
			
			methodSelect.addOption({value:m,label:m});
		}
	}
	
	var parametersTable = null;
	
	on(methodSelect,"change",function(m) {
		
		if (parametersTable) {
			parametersTable.destroy();
			parameterTable = null;
		}
		
		if (m) {

			parametersTable = new TableContainer({cols:1,showLabels:true});
			
			parametersTable.placeAt("parametersTable");
			
			var params = jsonRpcDemo._smd.services[m].parameters;
			
			for (var i=0;i<params.length;++i) {
				
				var param = params[i];
				
				var label = param.serialID ? (param.serialID + "(" + param.type +")") : param.type;
				
				parametersTable.addChild(
						new ValidationTextBox({required:!param.optional,label:label})
				);
				
			}
		
			parametersTable.startup();
		}
	});
	
	on(sendBtn,"click",function() {
		
		var m = methodSelect.get('value');
		
		if (m && parametersTable) {
			
			var mm = jsonRpcDemo[m];
			
			var params = jsonRpcDemo._smd.services[m].parameters;
			
			var children = parametersTable.getChildren();
			
			var args = [];
			
			for (var i=0;i<params.length;++i) {
				
				var param = params[i];
				var val = children[i].get('value');
				
				if (param.type == "object") {
					args.push(json.parse(val,true));
				}
				else if (param.type == "integer" || param.type == "number") {
					args.push(new Number(val));
				}
				else {
					args.push(val);
				}
			}

			var d = mm.apply(jsonRpcDemo,args);
			var resultDiv = dom.byId("resultDiv");
			
			domConstruct.empty(resultDiv);
			resultDiv.appendChild(document.createTextNode("Request "+m+" running..."));

			d.then(function(result) {
				
				domConstruct.empty(resultDiv);
				resultDiv.appendChild(document.createTextNode(json.stringify(result)));
			},
			function(err) {
				
				domConstruct.empty(resultDiv);
				resultDiv.appendChild(document.createTextNode(json.stringify(err)));
			});
		}
	});
});
});